# S3file #
__title__ = 'py-s3file'
__description__ = 'A simple utility tool to access s3 object through basic File I/O operations.'
__copywrite__ = 'Copyright (C) yhori991'
__version__ = '0.0.8'
__license__ = 'MIT'
__author__ = 'yhori991'
__author_email__ = 'yhori991@gmail.com'
__url__ = 'https://github.com/yhori991/py-s3file'

